# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinShare library

    @copyright: 2011, 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from ContentTypeSupport import getContentPreferences
from DateSupport import getCurrentTime
from MoinSupport import *
from MoinMoin import wikiutil
import re

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

escape = wikiutil.escape
_getFragments = getFragments

__version__ = "0.1"

# More Moin 1.9 compatibility functions.

def has_member(request, groupname, username):
    if hasattr(request.dicts, "has_member"):
        return request.dicts.has_member(groupname, username)
    else:
        return username in request.dicts.get(groupname, [])

# Fragments employ a "moinshare" attribute.

fragment_attribute = "moinshare"

def getFragments(s):

    "Return all fragments in 's' having the MoinShare fragment attribute."

    fragments = []
    for format, attributes, body in _getFragments(s):
        if attributes.has_key(fragment_attribute):
            fragments.append((format, attributes, body))
    return fragments

def getOutputTypes(request, format):

    """
    Using the 'request' and the 'format' of a fragment, return the media types
    available for the fragment.
    """

    # This uses an extended parser API method if available.

    parser = getParserClass(request, format)
    if hasattr(parser, "getOutputTypes"):
        return parser.getOutputTypes()
    else:
        return ["text/html"]

def getPreferredOutputTypes(request, mimetypes):

    """
    Using the 'request', perform content negotiation, obtaining mimetypes common
    to the fragment (given by 'mimetypes') and the client (found in the Accept
    header).
    """

    accept = getHeader(request, "Accept", "HTTP")
    if accept:
        prefs = getContentPreferences(accept)
        return prefs.get_preferred_types(mimetypes)
    else:
        return mimetypes

def getUpdatedTime(metadata):

    """
    Return the last updated time based on the given 'metadata', using the
    current time if no explicit last modified time is specified.
    """

    # NOTE: We could attempt to get the last edit time of a fragment.

    latest_timestamp = metadata.get("last-modified")
    if latest_timestamp:
        return latest_timestamp.as_ISO8601_datetime_string()
    else:
        return getCurrentTime().as_ISO8601_datetime_string()

def getUpdateSources(request, sources_page):

    """
    Using the 'request', return the update sources defined on the given
    'sources_page'.
    """

    # Remote sources are accessed via dictionary page definitions.

    return getWikiDict(sources_page, request)

# Entry/update classes.

class Update:

    "A feed update entry."

    def __init__(self):
        self.title = None
        self.link = None
        self.content = None
        self.content_type = None
        self.updated = None

        # Page-related attributes.

        self.fragment = None
        self.preferred = None

    def __cmp__(self, other):
        if self.updated is None and other.updated is not None:
            return 1
        elif self.updated is not None and other.updated is None:
            return -1
        else:
            return cmp(self.updated, other.updated)

# Update retrieval from pages.

def getUpdatesFromPage(page, request):

    """
    Get updates from the given 'page' using the 'request'. A list of update
    objects is returned.
    """

    updates = []

    # NOTE: Use the updated datetime from the page for updates.
    # NOTE: The published and updated details would need to be deduced from
    # NOTE: the page history instead of being taken from the page as a whole.

    metadata = getMetadata(page)
    updated = getUpdatedTime(metadata)

    # Get the fragment regions for the page.

    for n, (format, attributes, body) in enumerate(getFragments(page.get_raw_body())):

        # Produce a fragment identifier.
        # NOTE: Choose a more robust identifier where none is explicitly given.

        update = Update()

        update.fragment = attributes.get("fragment", str(n))
        update.title = attributes.get("summary", "Update #%d" % n)

        # Get the preferred content types available for the fragment.

        update.preferred = getPreferredOutputTypes(request, getOutputTypes(request, format))

        # Try and obtain some suitable content for the entry.
        # NOTE: Could potentially get a summary for the fragment.

        update.content = None

        if "text/html" in update.preferred:
            parser_cls = getParserClass(request, format)
            parser = parser_cls(body, request)

            if format == "html":
                update.content = body
            elif hasattr(parser, "formatForOutputType"):
                s = StringIO()
                parser.formatForOutputType("text/html", write=s.write)
                update.content = s.getvalue()
            else:
                fmt = request.html_formatter
                fmt.setPage(page)
                update.content = formatText(body, request, fmt, parser_cls)

            update.content_type = "html"

        update.link = page.url(request)
        update.updated = updated

        updates.append(update)

    return updates

# Source management.

def getUpdateSources(pagename, request):

    "Return the update sources from the given 'pagename' using the 'request'."

    sources = {}

    source_definitions = getWikiDict(pagename, request)

    if source_definitions:
        for name, value in source_definitions.items():
            sources[name] = getSourceParameters(value)

    return sources

def getSourceParameters(source_definition):

    "Return the parameters from the given 'source_definition' string."

    parameters = {}
    unqualified = ("type", "location")

    for arg in source_definition.split():
        try:
            argname, argvalue = arg.split("=", 1)

            # Detect unlikely parameter names.

            if not argname.isalpha():
                raise ValueError

            parameters[argname] = argvalue

        # Unqualified parameters are assumed to be one of a recognised set.

        except ValueError:
            for argname in unqualified:
                if not parameters.has_key(argname):
                    parameters[argname] = arg
                    break

    return parameters

# vim: tabstop=4 expandtab shiftwidth=4
